@@ -32,13 +32,15 @@
 #include <X11/Xatom.h>
 
 #include "panel-background-monitor.h"
-#include "panel-gdk-pixbuf-extensions.h"
+#include "panel-util.h"
 
 enum {
 	CHANGED,
 	LAST_SIGNAL
 };
 
+static void panel_background_monitor_changed (PanelBackgroundMonitor *monitor);
+
 static GdkFilterReturn panel_background_monitor_xevent_filter (GdkXEvent *xevent,
 							       GdkEvent  *event,
 							       gpointer   data);
@@ -68,7 +70,7 @@ struct _PanelBackgroundMonitor {
 	gboolean   display_grabbed;
 };
 
-static GObjectClass *parent_class;
+G_DEFINE_TYPE (PanelBackgroundMonitor, panel_background_monitor, G_TYPE_OBJECT);
 
 static PanelBackgroundMonitor **global_background_monitors = NULL;
 
@@ -83,6 +85,8 @@ panel_background_monitor_finalize (GObject *object)
 
 	gdk_window_remove_filter (
 		monitor->gdkwindow, panel_background_monitor_xevent_filter, monitor);
+	g_signal_handlers_disconnect_by_func (monitor->screen, 
+		panel_background_monitor_changed, monitor);
 
 	if (monitor->gdkpixmap)
 		g_object_unref (monitor->gdkpixmap);
@@ -92,7 +96,7 @@ panel_background_monitor_finalize (GObject *object)
 		g_object_unref (monitor->gdkpixbuf);
 	monitor->gdkpixbuf = NULL;
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (panel_background_monitor_parent_class)->finalize (object);
 }
 
 static void
@@ -100,8 +104,6 @@ panel_background_monitor_class_init (PanelBackgroundMonitorClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	signals [CHANGED] = 
 		g_signal_new ("changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -122,7 +124,7 @@ panel_background_monitor_init (PanelBackgroundMonitor *monitor)
 	monitor->gdkwindow = NULL;
 	monitor->xwindow   = None;
 
-	monitor->gdkatom = gdk_atom_intern ("_XROOTPMAP_ID", FALSE);
+	monitor->gdkatom = gdk_atom_intern_static_string ("_XROOTPMAP_ID");
 	monitor->xatom   = gdk_x11_atom_to_xatom (monitor->gdkatom);
 
 	monitor->gdkpixmap = NULL;
@@ -131,31 +133,6 @@ panel_background_monitor_init (PanelBackgroundMonitor *monitor)
 	monitor->display_grabbed = FALSE;
 }
 
-GType
-panel_background_monitor_get_type (void)
-{
-	static GType object_type = 0;
-
-	if (!object_type) {
-		static const GTypeInfo object_info = {
-			sizeof (PanelBackgroundMonitorClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) panel_background_monitor_class_init,
-			NULL,           /* class_finalize */
-			NULL,           /* class_data */
-			sizeof (PanelBackgroundMonitor),
-			0,              /* n_preallocs */
-			(GInstanceInitFunc) panel_background_monitor_init,
-		};
-
-		object_type = g_type_register_static (
-					G_TYPE_OBJECT, "PanelBackgroundMonitor", &object_info, 0);
-	}
-
-	return object_type;
-}
-
 static void
 panel_background_monitor_connect_to_screen (PanelBackgroundMonitor *monitor,
 					    GdkScreen              *screen)
@@ -167,6 +144,8 @@ panel_background_monitor_connect_to_screen (PanelBackgroundMonitor *monitor,
 	}
 
 	monitor->screen = screen;
+	g_signal_connect_swapped (screen, "size-changed", 
+	    G_CALLBACK (panel_background_monitor_changed), monitor);
 
 	monitor->gdkwindow = gdk_screen_get_root_window (screen);
 	monitor->xwindow   = gdk_x11_drawable_get_xid (monitor->gdkwindow);
@@ -220,6 +199,20 @@ panel_background_monitor_get_for_screen (GdkScreen *screen)
 	return g_object_ref (global_background_monitors [screen_number]);
 }
 
+static void
+panel_background_monitor_changed (PanelBackgroundMonitor *monitor)
+{
+	if (monitor->gdkpixmap)
+		g_object_unref (monitor->gdkpixmap);
+	monitor->gdkpixmap = NULL;
+
+	if (monitor->gdkpixbuf)
+		g_object_unref (monitor->gdkpixbuf);
+	monitor->gdkpixbuf = NULL;
+
+	g_signal_emit (monitor, signals [CHANGED], 0);
+}
+
 static GdkFilterReturn
 panel_background_monitor_xevent_filter (GdkXEvent *xevent,
 					GdkEvent  *event,
@@ -235,15 +228,8 @@ panel_background_monitor_xevent_filter (GdkXEvent *xevent,
 
 	if (xev->type == PropertyNotify &&
 	    xev->xproperty.atom == monitor->xatom &&
-	    xev->xproperty.window == monitor->xwindow) {
-	    	if (monitor->gdkpixmap)
-			g_object_unref (monitor->gdkpixmap);
-		monitor->gdkpixmap = NULL;
-		if (monitor->gdkpixbuf)
-			g_object_unref (monitor->gdkpixbuf);
-		monitor->gdkpixbuf = NULL;
-		g_signal_emit (monitor, signals [CHANGED], 0);
-	}
+	    xev->xproperty.window == monitor->xwindow)
+		panel_background_monitor_changed (monitor);
 
 	return GDK_FILTER_CONTINUE;
 }
@@ -309,17 +295,36 @@ panel_background_monitor_tile_background (PanelBackgroundMonitor *monitor,
 
 		gdk_pixbuf_fill (retval, pixel);
 	} else {
-		ArtIRect rect;
+		unsigned char   *data;
+		cairo_t         *cr;
+		cairo_surface_t *surface;
+		cairo_pattern_t *pattern;
 
-		rect.x0 = 0;
-		rect.y0 = 0;
-		rect.x1 = width;
-		rect.y1 = height;
+		data = g_malloc (width * height * 4);
+		if (!data)
+			return NULL;
 
-		panel_gdk_pixbuf_draw_to_pixbuf_tiled (
-			monitor->gdkpixbuf, retval,
-			rect, tilewidth, tileheight,
-			0, 0, 255, GDK_INTERP_NEAREST);
+		surface = cairo_image_surface_create_for_data (data,
+							       CAIRO_FORMAT_RGB24,
+							       width, height,
+							       width * 4);
+		cr = cairo_create (surface);
+		cairo_set_source_rgb (cr, 1, 1, 1);
+		cairo_paint (cr);
+
+		gdk_cairo_set_source_pixbuf (cr, monitor->gdkpixbuf, 0, 0);
+		pattern = cairo_get_source (cr);
+		cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+		cairo_rectangle (cr, 0, 0, width, height);
+		cairo_fill (cr);
+
+		cairo_destroy (cr);
+		cairo_surface_destroy (surface);
+
+		retval = panel_util_cairo_rgbdata_to_pixbuf (data,
+							     width, height);
+
+		g_free (data);
 	}
 
 	return retval;
@@ -350,8 +355,8 @@ panel_background_monitor_setup_pixbuf (PanelBackgroundMonitor *monitor)
 	gdk_drawable_get_size (
 		GDK_DRAWABLE (monitor->gdkpixmap), &pwidth, &pheight);
 
-	gdk_drawable_get_size (
-		GDK_DRAWABLE (monitor->gdkwindow), &rwidth, &rheight);
+	gdk_window_get_geometry (monitor->gdkwindow,
+				 NULL, NULL, &rwidth, &rheight, NULL);
 
 	monitor->width  = MIN (pwidth,  rwidth);
 	monitor->height = MIN (pheight, rheight);
